/*
Copyright (C) 1994-1995 Apogee Software, Ltd.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
/**********************************************************************
   module: RS.C

   Basic sound recorder. A modification of
   Simple sound player from James R. Dose.

   (c) Copyright 1995 James R. Dose.  All Rights Reserved.
**********************************************************************/

#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "blaster.h"
#include "dpmi.h"
#include "fx_man.h"

/*---------------------------------------------------------------------
   Function prototypes
---------------------------------------------------------------------*/

int   FileExists( const char *filename );
void  SaveFile( const char *filename, const char *buffer, int length );
char *GetUserText( const char *parameter );
int   CheckUserParm( const char *parameter );
void  Callback( char *ptr, int length );
void  CallbackEnd();
int   LockMemory();
void  UnlockMemory();

#define TRUE  ( 1 == 1 )
#define FALSE ( !TRUE )

#define NUMCARDS 4

char *SoundCardNames[] =
   {
   "Sound Blaster", "Awe 32", "Pro AudioSpectrum", "Sound Man 16"
   };

int SoundCardNums[] =
   {
   SoundBlaster, Awe32, ProAudioSpectrum, SoundMan16
   };

#define BUFFE
char *Samples;
volatile int SamplesFilled = 0;
int SamplesMax;

/*---------------------------------------------------------------------
   Function: main

   Sets up sound cards, calls the demo, and then cleans up.
---------------------------------------------------------------------*/

void main
   (
   int argc,
   char *argv[]
   )

   {
   int card;
   int voices;
   int rate;
   int bits;
   int channels;
   int status;
   fx_device device;
   char *ptr;
   char  filename[ 128 ];
   char ch;

   printf( "\nRS Build %s %s\n", __DATE__, __TIME__ );

   if ( ( CheckUserParm( "?" ) ) || ( argc < 2 ) )
      {
      int index;

      printf(
         "Usage: RS [ rawsoundfile ] CARD=[ card # ]\n"
         "          RATE=[ capture rate ] MAXSAMPLES=[ max samples ]\n\n"
         "   sound card # = \n" );
      for( index = 0; index < NUMCARDS; index++ )
         {
         printf( "      %d : %s\n", index, SoundCardNames[ index ] );
         }

      printf( "\nDefault: RS [ rawsoundfile ] CARD=0 RATE=11025 MAXSAMPLES=44100\n\n" );
      exit( 0 );
      }

   // Default is Sound Blaster
   card       = 0;
   voices     = 1;
   bits       = 8;
   channels   = 1;
   rate       = 11025;
   SamplesMax = 44100;

   ptr = GetUserText( "MAXSAMPLES" );
   if ( ptr != NULL )
      {
      sscanf( ptr, "%d", &SamplesMax );
      }

   ptr = GetUserText( "RATE" );
   if ( ptr != NULL )
      {
      sscanf( ptr, "%d", &rate );
      }

   ptr = GetUserText( "CARD" );
   if ( ptr != NULL )
      {
      sscanf( ptr, "%d", &card );
      }

   if ( ( card < 0 ) || ( card >= NUMCARDS ) )
      {
      printf( "Value out of range for sound card #: %d\n", card );
      exit( 1 );
      }

   strcpy( filename, argv[ 1 ] );
   if ( FileExists( filename ) )
      {
      printf( "\nAre you sure you want to overwrite the file? Y/N\n" );
      switch ( getch() )
         {
         case 'Y':
         case 'y':
            break;
         default:
            exit( 1 );
         }
      printf( "\n" );
      }

   Samples = ( char * )malloc( SamplesMax );
   if ( Samples == NULL )
      {
      printf( "Out of memory while allocating %d bytes.\n", SamplesMax );
      exit( 1 );
      }

   card = SoundCardNums[card];
   if ((card == SoundBlaster) || (card == Awe32))
      {
      BLASTER_CONFIG Config;
      status = BLASTER_GetEnv( &Config );
      if ( status != BLASTER_Ok )
         {
         printf( "%s\n", BLASTER_ErrorString( status ) );
         exit( 1 );
         }
      BLASTER_SetCardSettings( Config );
      }

   status = FX_SetupCard( card, &device );
   if ( status != FX_Ok )
      {
      printf( "%s\n", FX_ErrorString( status ) );
      exit( 1 );
      }

   status = FX_Init( card, voices, channels, bits, rate );
   if ( status != FX_Ok )
      {
      printf( "%s\n", FX_ErrorString( status ) );
      exit( 1 );
      }

   FX_SetVolume( 255 );

   status = LockMemory();

   if ( status != DPMI_Ok )
      {
      UnlockMemory();
      printf( "Memory locking failed\n" );
      exit( 1 );
      }

   printf( "Recording sound to be written to file '%s'.\n\n", filename );
   printf( "Press ESCape to end.\n\n" );

   FX_StartRecording( rate, Callback );

   ch = 0;
   while(( ch != 27 ) && ( SamplesFilled < SamplesMax ))
      {
      if (kbhit())
         {
         ch = getch();
         }
      }

   FX_StopRecord();
   FX_Shutdown();

   UnlockMemory();
   SaveFile( filename, Samples, SamplesFilled );
   free (Samples);

   printf( "\n" );
   }


/*---------------------------------------------------------------------
   Function: FileExists

   Checks if a file exists.
---------------------------------------------------------------------*/

int FileExists
   (
   const char *filename
   )

   {
   FILE   *in;

   if ( ( in = fopen( filename, "rb" ) ) == NULL )
      {
      return 0;
      }

   fclose(in);
   return 1;
   }

/*---------------------------------------------------------------------
   Function: SaveFile

   Writes data to file.
---------------------------------------------------------------------*/

void SaveFile
   (
   const char *filename,
   const char *buffer,
   int        length
   )

   {
   FILE   *out;

   if ( ( out = fopen( filename, "wb" ) ) == NULL )
      {
      return;
      }

   fwrite(buffer, length, 1, out);
   fclose(out);
   }


/*---------------------------------------------------------------------
   Function: GetUserText

   Checks if the specified string is present in the command line
   and returns a pointer to the text following it.
---------------------------------------------------------------------*/

char *GetUserText
   (
   const char *parameter
   )

   {
   int i;
   int length;
   char *text;
   char *ptr;

   extern int   _argc;
   extern char **_argv;

   text = NULL;
   length = strlen( parameter );
   i = 1;
   while( i < _argc )
      {
      ptr = _argv[ i ];

      if ( ( strnicmp( parameter, ptr, length ) == 0 ) &&
         ( *( ptr + length ) == '=' ) )
         {
         text = ptr + length + 1;
         break;
         }

      i++;
      }

   return( text );
   }


/*---------------------------------------------------------------------
   Function: CheckUserParm

   Checks if the specified string is present in the command line.
---------------------------------------------------------------------*/

int CheckUserParm
   (
   const char *parameter
   )

   {
   int i;
   int found;
   char *ptr;

   extern int   _argc;
   extern char **_argv;

   found = FALSE;
   i = 1;
   while( i < _argc )
      {
      ptr = _argv[ i ];

      // Only check parameters preceded by - or /
      if ( ( *ptr == '-' ) || ( *ptr == '/' ) )
         {
         ptr++;
         if ( stricmp( parameter, ptr ) == 0 )
            {
            found = TRUE;
            break;
            }
         }

      i++;
      }

   return( found );
   }


/*---------------------------------------------------------------------
   Function: Callback

   Passed to FX_StartRecording.
   Called when recorded samples are available.
---------------------------------------------------------------------*/

void Callback
   (
   char *ptr,
   int  length
   )

   {
   if (SamplesFilled + length <= SamplesMax)
      {
      for (; length--; Samples[SamplesFilled++] = *ptr++)
         ;
      }
   else if (SamplesFilled < SamplesMax)
      {
      for (; SamplesFilled < SamplesMax; Samples[SamplesFilled++] = *ptr++)
         ;
      }
   }

/*---------------------------------------------------------------------
   Function: CallbackEnd

   Used for determining the length of the function to lock in memory.
---------------------------------------------------------------------*/

void CallbackEnd
   (
   void
   )

   {
   }

/*---------------------------------------------------------------------
   Function: LockMemory

   Locks all necessary data.
---------------------------------------------------------------------*/

int LockMemory
   (
   void
   )

   {
   int status;

   status  = DPMI_LockMemoryRegion( Callback, CallbackEnd );
   status |= DPMI_Lock( Samples );
   status |= DPMI_Lock( (int)SamplesFilled );
   status |= DPMI_Lock( SamplesMax );

   return status;
   }

/*---------------------------------------------------------------------
   Function: UnlockMemory

   Unlocks all necessary data.
---------------------------------------------------------------------*/

void UnlockMemory
   (
   void
   )

   {
   DPMI_UnlockMemoryRegion( Callback, CallbackEnd );
   DPMI_Unlock( Samples );
   DPMI_Unlock( (int)SamplesFilled );
   DPMI_Unlock( SamplesMax );
   }
